if 'INSTALLED_APPS' not in locals()["__builtins__"]:
    INSTALLED_APPS = []

INSTALLED_APPS += (
    'pinax.eventlog',  # Not mandatory
    'bootstrapform',
    'django.contrib.staticfiles',
    'django_tables2',
    'background_task',
    'django_tables2_reports',
    'django_excel_to_model',
)

# TEMPLATE_CONTEXT_PROCESSORS += (
#     "django.core.context_processors.request",
#     'django.core.context_processors.static',
# )



from django_excel_to_model.management.commands.utils.counter import Counter


class BulkInserter(object):
    def __init__(self, model, max_insert_number=5000):
        super(BulkInserter, self).__init__()
        self.max_insert_number = max_insert_number
        self.model = model
        self.count = Counter(max_insert_number, notification_interval=None)
        self.obj_list = []

    def insert(self, obj_dict):
        self.count.decrease()
        self.obj_list.append(self.model(**obj_dict))
        if self.count.is_equal_or_below(0):
            self.commit()
            self.count.reset()

    def commit(self):
        self.model.objects.bulk_create(self.obj_list)
        self.obj_list = []





class DataFileBase(object):
    def get_sheet(self, index):
        raise Exception("Not implemented")


class ExcelBaseFile(object):
    def __init__(self, full_path):
        raise Exception("Failed to get open workbook")

    def get_sheet(self, index):
        raise Exception("Failed to get sheet")





class SingleInserter(object):

    def __init__(self, model):
        super(SingleInserter, self).__init__()
        self.model = model

    def insert(self, item_info_dict):
        target, is_created = self.model.objects.get_or_create(**item_info_dict)
        if not is_created:
            print "item duplicated:", item_info_dict

    def commit(self):
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_django-excel-to-model
------------

Tests for `django-excel-to-model` models module.
"""

from django.test import TestCase

from django_excel_to_model import models


class TestDjango_excel_to_model(TestCase):

    def setUp(self):
        pass

    def test_something(self):
        pass

    def tearDown(self):
        pass



from django_excel_to_model.field_tools import get_db_field
from django_excel_to_model.file_readers.data_source_base import BaseSheet


class XlsbSheet(BaseSheet):
    def __init__(self, workbook, index=0):
        self.workbook = workbook
        self.sheet = workbook.get_sheet(index+1)
        self.title_columns = None

    def set_header_row(self, header_row_start_from_0):
        self.title_columns = self.get_columns(header_row_start_from_0)

    def _format_number(self, cell):
        if cell.v == int(cell.v):
            return int(cell.v)
        return cell.v

    def parse_cell_value(self, cell):
        try:
            final_value = int(cell.v)
        except ValueError:
            final_value = cell.v
        return final_value

    def get_mapped_columns(self, row_index, mapping):
        for mapped_data in self.enumerate_mapped(mapping, row_index):
            return mapped_data

    def enumerate_mapped(self, mapping, start_row_numbered_from_0=1):
        for row in self.sheet.rows(sparse=True):
            res = {}
            for cell in row:
                if cell.v is not None and cell.r < start_row_numbered_from_0:
                    break
                if cell.v is not None and cell.v not in self.title_columns:
                    src_key = unicode(self.title_columns[cell.c])
                    target_key = get_db_field(mapping, src_key)
                    if target_key:
                        res[target_key] = self.parse_cell_value(cell)
            yield res

    def get_columns(self, row_index):
        res = []
        for row in self.sheet.rows(sparse=True):
            for r in row:
                if r.r == row_index and r.v is not None:
                    res.append((str(r.v)))
        return res



from ufs_tools.short_decorator.ignore_exception import ignore_exc, ignore_exc_with_result

from django_excel_to_model.file_readers.csv_reader import CsvFile
from django_excel_to_model.openpyxl_reader import OpenpyxlExcelFile
from django_excel_to_model.reader import XlsbFile, ExcelFile


class DataSourceFactory(object):
    def __init__(self, full_path):
        """
        :param full_path: full path of the data file
        """
        super(DataSourceFactory, self).__init__()
        self.full_path = full_path

    def get_data_source(self, sheet_index_numbered_from_0, header_row_start_from_0):
        """
        :param sheet_index_numbered_from_0: integer, index of the sheet
        :param header_row_start_from_0:
        :return: DataSourceBase object
        """
        for excel_file_reader in [self.get_xlrd_file,
                                  self.get_openpyxl_file, self.get_xlsb_file]:
            excel_file = excel_file_reader()
            if excel_file:
                sheet = excel_file.get_sheet(sheet_index_numbered_from_0)
                sheet.set_header_row(header_row_start_from_0)
                return sheet

        return CsvFile(self.full_path)

    @ignore_exc_with_result(is_notification_needed=True)
    def get_xlrd_file(self):
        return ExcelFile(self.full_path)

    @ignore_exc_with_result(is_notification_needed=True)
    def get_openpyxl_file(self):
        return OpenpyxlExcelFile(self.full_path)

    @ignore_exc_with_result(is_notification_needed=True)
    def get_xlsb_file(self):
        return XlsbFile(self.full_path)




from django.conf.urls import url
from django_excel_to_model.views_import import upload_file
from views import create_mapping

urlpatterns = [
    # (r'^translate/',
    #  ExcelMappingGeneratorView.as_view(template_name=
    #                                    'excel_reader/excel_mapping_generator.html')),
    url(r'^$', create_mapping),
    url(r'^upload/', upload_file),
]



import argparse
import importlib
import os
from django.contrib.contenttypes.models import ContentType
from django.core.management.base import BaseCommand

from django_excel_to_model.field_tools import get_valid_excel_field_name
from django_excel_to_model.file_readers.csv_reader import CsvFile
from django_excel_to_model.file_readers.data_source_factory import DataSourceFactory
from django_excel_to_model.management.commands.utils.bulk_inserter import BulkInserter
from django_excel_to_model.management.commands.utils.counter import Counter
from django_excel_to_model.models import ExcelImportTask
from django_excel_to_model.reader import ExcelFile, XlsbFile

try:
    from pinax.eventlog.models import log
except ImportError:
    log = None


class MandatoryColumnMissing(Exception):
    pass


class DictTranslator(object):
    # noinspection PyMethodMayBeStatic
    def translate(self, item_dict):
        for ignored in ["__invalid"]:
            if ignored in item_dict:
                del item_dict["__invalid"]
        return item_dict


class ExcelFileFromClassImporter(object):
    def __init__(self, class_instance, sheet_numbered_from_1=1):
        super(ExcelFileFromClassImporter, self).__init__()
        self.model_module = importlib.import_module(class_instance.__module__)
        self.class_instance = class_instance
        self.translator = DictTranslator()
        self.sheet_numbered_from_1 = sheet_numbered_from_1
        self.inserter = BulkInserter(self.class_instance)
        self.mandatory_column_headers = None
        self.is_clean_before_import = False

    def set_clean_before_import(self):
        self.is_clean_before_import = True

    def import_excel(self, full_path, header_row_numbered_from_1=1, first_import_row_numbered_from_1=2, count=1000):
        filename = os.path.basename(full_path)
        data_source = DataSourceFactory(full_path).get_data_source(
            sheet_index_numbered_from_0=self.sheet_numbered_from_1 - 1,
            header_row_start_from_0=header_row_numbered_from_1 - 1)
        count = min(data_source.get_total_rows(), count)
        c = Counter(count)

        self.validate_existence_of_mandatory_columns(data_source)

        if self.is_clean_before_import:
            self.class_instance.objects.all().delete()

        column_to_db_field_mapping = self._get_column_to_db_field_mapping(data_source)

        for item_info_dict in data_source.enumerate_mapped(column_to_db_field_mapping,
                                                           start_row_numbered_from_0=first_import_row_numbered_from_1 - 1):
            # print item_info_dict
            self.translator.translate(item_info_dict)
            item_info_dict["data_import_id"] = filename
            self.inserter.insert(item_info_dict)
            # If count = 1, when 1 processed, cnt will become 0
            c.decrease()
            if c.is_equal_or_below(0):
                self.commit_and_log(filename)
                return 0
        self.commit_and_log(filename)
        return -1

    def _get_column_to_db_field_mapping(self, data_source):
        column_to_db_field_mapping = {}
        for column_name in data_source.get_headers():
            if column_name in self.model_module.mapping:
                column_to_db_field_mapping[column_name] = self.model_module.mapping[column_name]
            else:
                converted_field_name = get_valid_excel_field_name(column_name)
                if converted_field_name in self.model_module.mapping:
                    column_to_db_field_mapping[column_name] = \
                        self.model_module.mapping[converted_field_name]
        return column_to_db_field_mapping

    def validate_existence_of_mandatory_columns(self, sheet):
        if self.mandatory_column_headers is not None:
            if not all(spreadsheet_column_header in sheet.get_headers()
                       for spreadsheet_column_header in self.mandatory_column_headers):
                raise MandatoryColumnMissing()

    def commit_and_log(self, filename):
        self.inserter.commit()
        if log is not None:
            log(
                user=None,
                action=filename,
                extra={
                    "filename": filename
                },
                obj=ContentType.objects.get_for_model(ExcelImportTask)
            )


def import_excel_according_to_model(full_path, content_type_id, header_row_numbered_from_1,
                                    first_import_row_numbered_from_1, count=1000):
    content = ContentType.objects.get(pk=content_type_id)
    e = ExcelFileFromClassImporter(content.model_class())
    e.import_excel(full_path, header_row_numbered_from_1, first_import_row_numbered_from_1, count)


class Command(BaseCommand):
    help = 'Import excel according to model info'

    def add_arguments(self, parser):
        parser.add_argument('file-path')
        parser.add_argument('content-type-id')
        parser.add_argument('header_row_numbered_from_1')
        parser.add_argument('start')
        parser.add_argument('count', nargs='?')

    def handle(self, *args, **options):
        parser = argparse.ArgumentParser(description='Import excel file according to model info')

        subparsers = parser.add_subparsers(help='sub-command help')

        # create the parser for the "a" command
        parser_import_excel_according_to_model = subparsers.add_parser(
            'import_excel_according_to_model', help='import_excel_according_to_model help')

        # parser_import_excel_according_to_model.add_argument(
        #     '--content-type-id', type=int, help='content type pk')
        # parser_import_excel_according_to_model.add_argument(
        #     '--start', default=1,
        #     help='start line for importing excel, default=1 (second line)')
        parser_import_excel_according_to_model.add_argument(
            'file-path', nargs=1, help='path of the excel file')
        parser_import_excel_according_to_model.add_argument(
            'content-type-id', nargs=1, help='content id of the model', type=int)
        parser_import_excel_according_to_model.add_argument(
            'header_row_numbered_from_1', nargs=1, help='header row number (start from 1)', type=int)
        parser_import_excel_according_to_model.add_argument(
            'start', nargs=1, help='path of the excel file', type=int)
        parser_import_excel_according_to_model.add_argument(
            'count', nargs="?", help='process line count', default=[1000], type=int)

        arg_result = parser.parse_args()
        print(vars(arg_result)["file-path"][0])
        print(vars(arg_result)["content-type-id"][0])
        print(vars(arg_result)["start"][0])
        print(vars(arg_result)["count"])

        file_path = vars(arg_result)["file-path"][0]
        content_type_id = vars(arg_result)["content-type-id"][0]
        header_row_numbered_from_1 = vars(arg_result)["header_row_numbered_from_1"][0]
        first_import_row_numbered_from_1 = vars(arg_result)["start"][0]
        count = vars(arg_result)["count"]

        return import_excel_according_to_model(
            file_path, content_type_id, header_row_numbered_from_1, first_import_row_numbered_from_1, count)



import pinyin

from django_excel_to_model.file_readers.file_reader_exceptions import NonUnicodeFieldNameNotSupported

SPECIFIC_CHAR_MAPPING = {
    "%" : 'percentage'
}

def get_target_field_name(col):
    for ch in [" ", ",", "_", ")", "(", ":", "/", "\\", '"', "'", "-", ",", ".", "<", ">", "%", "&", "\r", "\n"]:
        if ch in SPECIFIC_CHAR_MAPPING:
            col = col.replace(ch, SPECIFIC_CHAR_MAPPING[ch])
        else:
            col = col.replace(ch, "_").replace("__", "_")
    # col = filter(lambda x: x in string.printable, col)
    col = get_string_with_only_char_in_list(col)
    col = col.replace("__", "_")
    col = col.strip("_")
    if col == "id":
        return "original_id"
    if col == "global":
        return "global_item"
    return col


var_name_char_list = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_"


def get_string_with_only_char_in_list(col, char_list=var_name_char_list):
    col = pinyin.get(col, format="strip")
    res = u""
    for ch in col:
        if ch in char_list:
            res += ch
    col = res
    return col


def get_valid_excel_field_name(col):
    """
    This function will not return the exact string!!!!!!
    :param col:
    :return:
    """
    #if not (type(col) is unicode):
    #    raise NonUnicodeFieldNameNotSupported
    # Only ascii supported here
    return get_string_with_only_char_in_list(col, var_name_char_list + " \t")


def get_db_field(mapping, src_key):
    if src_key in mapping:
        target_key = mapping[src_key]
    elif get_valid_excel_field_name(src_key) in mapping:
        target_key = mapping[get_valid_excel_field_name(src_key)]
    else:
        target_key = None
    return target_key





class Counter(object):
    def __init__(self, init=0, notification_interval=1000):
        super(Counter, self).__init__()
        self.cnt = init
        self.init = init
        self.notification_interval = notification_interval
        if self.notification_interval:
            print("Start counting: %d/%d" % (self.init-self.cnt, self.init))

    def decrease(self):
        self.cnt -= 1
        if self.notification_interval and ((self.init-self.cnt) % self.notification_interval == 0):
            print("Current: %d/%d" % (self.init-self.cnt, self.init))

    def is_equal_or_below(self, value):
        return self.cnt <= value

    def reset(self):
        self.cnt = self.init



import io

from django_excel_to_model.field_tools import get_valid_excel_field_name
from django_excel_to_model.reader import ExcelBaseFile


class CsvFile(ExcelBaseFile):
    def __init__(self, full_path):
        self.full_path = full_path
        self.sheet_index = None
        self.title_columns = None
        self.title_column_len = None

    def get_sheet(self, index):
        self.sheet_index = index
        return self

    def get_headers(self, header_index=0):
        header = None
        with io.open(self.full_path, "r", encoding="utf_8_sig", newline='\r\n') as f:
            for idx, line in enumerate(f):
                if idx == header_index:
                    header = line
                    break
        self.title_columns = header.split(';')
        assert self.title_columns is not None
        self.title_column_len = len(self.title_columns)

    def enumerate_mapped(self, mapping, start_row=1):
        with io.open(self.full_path, "r", encoding="utf_8_sig", newline='\r\n') as f:
            it = next(f)
            for line in f:
                yield self.get_mapped_columns(line, mapping)

    def get_mapped_columns(self, line, mapping):

        value_list = line.split(';')
        if len(self.title_columns) != len(value_list):
            raise Exception('Line length is not consistent with header: %s' % line)

        res = {}
        for i in range(self.title_column_len):
            src_key = unicode(self.title_columns[i])
            try:
                mapping_key = mapping[src_key]
            except KeyError:
                mapping_key = mapping[get_valid_excel_field_name(src_key)]
            val = value_list[i]
            res[mapping_key] = val
        return res



from django_excel_to_model.field_tools import get_db_field


class DataSourceBase(object):

    def __init__(self):
        super(DataSourceBase, self).__init__()
        self._headers = None

    def get_headers(self, row_index_start_from_0=None):
        """
        :param row_index_start_from_0: header row of the data source, start from 0
        :return: list of header column names
        """
        if self._headers:
            return self._headers
        else:
            raise Exception("No header row index given, call set_header_row before calling this function")

    def enumerate(self, start_row_numbered_from_0=1):
        """
        Enumerate rows in data source. Row data will be yield in a list of column values (in unicode)
        :param start_row_numbered_from_0:
        :return: A generator for rows. The format should be as RowRandomlyAccessibleSheet.get_row()
        """
        raise Exception("Not Implemented")

    def enumerate_mapped(self, mapping, start_row_numbered_from_0=1):
        headers = self.get_headers()
        for row in self.enumerate(start_row_numbered_from_0):
            res = {}
            col_index = 0
            for column in row:
                src_key = headers[col_index]
                target_key = get_db_field(mapping, src_key)
                if target_key:
                    res[target_key] = column
                col_index += 1
            yield res

    def get_total_rows(self):
        raise Exception("Not Implemented")


class BaseSheet(DataSourceBase):
    def __init__(self, workbook, index):
        self.workbook = workbook
        self.sheet = None
        self.title_columns = None

    def set_header_row(self, header_row_start_from_0):
        raise Exception("Not Implemented")

    def parse_cell_value(self, cell):
        print ('get cell value according to cell type')


class RowRandomlyAccessibleSheet(DataSourceBase):
    def __init__(self):
        super(RowRandomlyAccessibleSheet, self).__init__()
        self._headers = None
        # self.header_row_start_from_0 = None

    def set_header_row(self, header_row_start_from_0):
        self._headers = []
        for column in self.get_row(header_row_start_from_0):
            self._headers.append(column)

    def get_row(self, row_index_start_from_0):
        """
        :param row_index_start_from_0:
        :return: One row will contain a list of unicode string or int or date.
                E.g. [u"Value for Column1", "Value for Column2", 1, ...]
        """
        raise Exception("Not Implemented")



# -*- coding: utf-8 -*-
from datetime import datetime
import pytz
import pyxlsb
from xlrd import open_workbook, XL_CELL_DATE, xldate_as_tuple, XL_CELL_NUMBER
from django.utils import timezone
from django_excel_to_model.file_readers.data_file_base import ExcelBaseFile
from django_excel_to_model.file_readers.data_source_base import DataSourceBase, BaseSheet, RowRandomlyAccessibleSheet
from django_excel_to_model.xlsb_reader import XlsbSheet


class Sheet(RowRandomlyAccessibleSheet):
    def __init__(self, workbook, index=0):
        super(Sheet, self).__init__()
        self.workbook = workbook
        self.sheet = workbook.sheet_by_index(index)
        self.title_columns = None
        self.formatting_info = True

    def disable_formatting_info(self):
        self.formatting_info = False

    def _format_number(self, cell):
        if self.formatting_info:
            xf = self.workbook.xf_list[cell.xf_index]
            fmt_key = xf.format_key
            fmt = self.workbook.format_map[fmt_key]
            if fmt.format_key == 0:
                # Its format is General, so no additional fractional part should be displayed
                if cell.value == float(int(cell.value)):
                    return int(cell.value)
        elif cell.value == int(cell.value):
            return int(cell.value)
        return cell.value

    def _get_format_info(self, cell):
        print ("cell.xf_index is", cell.xf_index)
        fmt = self.workbook.xf_list[cell.xf_index]
        print ("type(fmt) is", type(fmt))
        print ("fmt.dump():")
        fmt.dump()

    def get_total_rows(self):
        return self.sheet.nrows

    def parse_cell_value(self, cell):
        if cell.ctype == XL_CELL_DATE:
            date_for_cell = xldate_as_tuple(cell.value, 0)
            native_date = datetime(date_for_cell[0], date_for_cell[1], date_for_cell[2], date_for_cell[3],
                                   date_for_cell[4], date_for_cell[5])
            try:
                time_zone_aware = timezone.make_aware(native_date, timezone.get_current_timezone())
            except OverflowError:
                time_zone_aware = native_date.replace(tzinfo=pytz.UTC)
            final_value = time_zone_aware
        elif cell.ctype == XL_CELL_NUMBER:
            # self.get_format_info(cell)
            final_value = self._format_number(cell)
        else:
            final_value = cell.value
        return final_value

    def print_headers(self):
        self.get_headers()
        for column in self.title_columns:
            print(column)

    def get_row(self, row_index_start_from_0):
        """
        Return row as defined in the interface.
        cell is got from self.sheet.cell(row_index_numbered_from_0, col_index_numbered_from_0)
        :param row_index_start_from_0:
        :return:
        """
        row_data = []
        for col_index in range(self.get_total_columns()):
            row_data.append(self.parse_cell_value(self.sheet.cell(row_index_start_from_0, col_index)))
        return row_data

    def enumerate(self, start_row_numbered_from_0=1):
        for row_index_numbered_from_0 in range(start_row_numbered_from_0, self.get_total_rows()):
            yield self.get_row(row_index_numbered_from_0)

    def get_total_columns(self):
        return self.sheet.ncols


class SheetConfig(object):
    def __init__(self):
        self.__has_header = True

    def has_header(self):
        return self.__has_header


class ExcelFile(ExcelBaseFile):
    def __init__(self, full_path):
        self.workbook = open_workbook(full_path)

    def get_sheet(self, index):
        s = Sheet(self.workbook, index)
        s.disable_formatting_info()
        return s


class XlsbFile(ExcelBaseFile):
    def __init__(self, full_path):
        self.workbook = pyxlsb.open_workbook(full_path)

    def get_sheet(self, index):
        sheet = XlsbSheet(self.workbook, index)
        return sheet



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_django-excel-to-model
------------

Tests for `django-excel-to-model` models module.
"""

from django.test import TestCase
# from unittest import TestCase

from django_excel_to_model.openpyxl_reader import OpenpyxlExcelFile
from sap_asset_master_data20191224.models import mapping


class TestOpenpyxl(TestCase):

    def setUp(self):
        pass

    def test_something(self):
        x = OpenpyxlExcelFile(r"C:\N-PC0WN7R6-Data\q19420\Downloads\sapItems20191223-1.XLSx")
        s = x.get_sheet(0)
        s.set_header_row(0)
        for i in s.enumerate_mapped(mapping, 2):
            print(i)

    def tearDown(self):
        pass



from openpyxl import Workbook, load_workbook

from django_excel_to_model.file_readers.data_source_base import DataSourceBase, RowRandomlyAccessibleSheet
from django_excel_to_model.reader import BaseSheet


# Ref: https://www.osgeo.cn/openpyxl/tutorial.html
# Ref: https://zhuanlan.zhihu.com/p/51292549
class XlsxSheetWithOpenOpenpyxl(RowRandomlyAccessibleSheet):
    def __init__(self, workbook, index):
        super(XlsxSheetWithOpenOpenpyxl, self).__init__()
        self.headers = None
        self.sheet = workbook[workbook.sheetnames[index]]
        self.header_row_start_from_0 = None

    def enumerate(self, start_row_numbered_from_0=1):
        for row in self.sheet.rows[start_row_numbered_from_0:]:
            row_data = self.raw_data_to_row(row)
            if row_data.count(None) == len(row_data):
                continue
            yield row_data

    def get_row(self, row_index_start_from_0):
        return self.raw_data_to_row(self.sheet.rows[row_index_start_from_0])

    def raw_data_to_row(self, raw_row_data):
        """

        :param raw_row_data: raw data read from openpyxl.
                        column.has_style    column.number_format    value type
              date:     True                'mm-dd-yy'              datetime
              ?float:   True                '0.00'                  float
              int:      True                '0.00'                  long
              str:      False               '0.00'                  unicode
        :return:
        """
        row = []
        for column in raw_row_data:
            row.append(column.value)
        return row

    def get_total_rows(self):
        return len(self.sheet.rows)


class OpenpyxlExcelFile(object):
    def __init__(self, full_path, flag=None):
        super(OpenpyxlExcelFile, self).__init__()
        self.wb = load_workbook(full_path)

    def get_sheet(self, sheet_index_numbered_from_0):
        return XlsxSheetWithOpenOpenpyxl(self.wb, sheet_index_numbered_from_0)
